/**
 * @private Internal code, does not follow semantic versioning.
 */
export declare const paramConnectVersion = "connect";
export declare const paramEncoding = "encoding";
export declare const paramCompression = "compression";
export declare const paramBase64 = "base64";
export declare const paramMessage = "message";
