/**
 * @private Internal code, does not follow semantic versioning.
 */
export declare const headerContentType = "Content-Type";
export declare const headerEncoding = "Grpc-Encoding";
export declare const headerAcceptEncoding = "Grpc-Accept-Encoding";
export declare const headerTimeout = "Grpc-Timeout";
export declare const headerGrpcStatus = "Grpc-Status";
export declare const headerGrpcMessage = "Grpc-Message";
export declare const headerStatusDetailsBin = "Grpc-Status-Details-Bin";
export declare const headerMessageType = "Grpc-Message-Type";
export declare const headerUserAgent = "User-Agent";
