/**
 * @private Internal code, does not follow semantic versioning.
 */
export declare const headerContentType = "Content-Type";
export declare const headerUnaryContentLength = "Content-Length";
export declare const headerUnaryEncoding = "Content-Encoding";
export declare const headerStreamEncoding = "Connect-Content-Encoding";
export declare const headerUnaryAcceptEncoding = "Accept-Encoding";
export declare const headerStreamAcceptEncoding = "Connect-Accept-Encoding";
export declare const headerTimeout = "Connect-Timeout-Ms";
export declare const headerProtocolVersion = "Connect-Protocol-Version";
export declare const headerUserAgent = "User-Agent";
